/***************************************************************************
    NWNXFunction.cpp - Implementation of the CNWNXFunction class.
    Copyright (C) 2003 Ingmar Stieger (Papillon)
    email: papillon@blackdagger.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ***************************************************************************/

#include "stdafx.h"
#include "NWNXFunction.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CNWNXFunction::CNWNXFunction()
{
}

CNWNXFunction::~CNWNXFunction()
{
}

BOOL CNWNXFunction::OnCreate (const char* LogDir)
{
	// we do not need a log file, so just exit
	return true;
}

void CNWNXFunction::SetLockDC(char* value)
{
	if (*(pGameObject+0x8) == 0x9) // object type placeable
	{
		int iLockDC = atoi(value);
		*(pGameObject+0x210) = iLockDC;
	}
}


void CNWNXFunction::SetGoldPieceValue(char* value)
{
	if (*(pGameObject+0x8) == 0x6) // object type item
	{
		int iGoldValue = atoi(value);
		*(int*)(pGameObject+0x1F4) = iGoldValue;
	}
}

void CNWNXFunction::SetTag(char* value)
{
    char* tag = (char*)(*(int*)(pGameObject+0x18));

	int iLength;
	int iOrgLength = strlen(tag);
	int iNewLength = strlen(value);

	if (iOrgLength > iNewLength)
		iLength = iNewLength;
	else
		iLength = iOrgLength;

	strncpy(tag, value, iLength);

	*(tag+iLength) = 0x0;
}

void CNWNXFunction::SetArmorAC(char* value)
{
	if (*(pGameObject+0x8) == 0x6) // object type item
	{
		int iAC = atoi(value);
		*(int*)(pGameObject+0x1CC) = iAC;
	}
	// 0x1d4: auch armor class ??
}

void CNWNXFunction::GetArmorAC(char* value)
{
	if (*(pGameObject+0x8) == 0x6) // object type item
	{
		int iAC = *(int*)(pGameObject+0x1CC);
		if (strlen(value) > 1)
			sprintf(value, "%d", iAC);
	}
}

void CNWNXFunction::GetDescription(char* value)
{
	int length = strlen(value);
	char* desc;
	
	if (*(pGameObject+0x8) != 0x6) // object type item	
		return;
	
	desc = (char*)(*(int*)(pGameObject+0x228));
	if (*(desc+4) == 0)
		return;
	desc = (char*)(*(int*)(desc));
	desc = (char*)(*(int*)(desc));
	desc = (char*)(*(int*)(desc+0x8));
	desc = (char*)(*(int*)(desc+0x4));

	int iDescLength = strlen(desc);
	if (iDescLength < length)
	{
		strncpy(value, desc, iDescLength);
		*(value+iDescLength) = 0x0;
	}
	else
	{
		strncpy(value, desc, length);
		*(value+length) = 0x0;
	}
}

void CNWNXFunction::GetIsPickPocketable(char* value)
{
	if (*(pGameObject+0x8) == 0x6) // object type item
	{
		int i = *(int*)(pGameObject+0x248);
		if (strlen(value) > 1)
			sprintf(value, "%d", i);
	}
}

void CNWNXFunction::SetIsPickPocketable(char* value)
{
	if (*(pGameObject+0x8) == 0x6) // object type item
	{
		int i = atoi(value);
		*(int*)(pGameObject+0x248) = i;
	}
}

void CNWNXFunction::SetBodyPart(char *value) 
{
	int bpart,ptype;
	char *next;

	if (*(pGameObject+0x8) == 0x5) // object type creature
	{
		// parse "nPart[!nType]"
		bpart = atoi(value);
		next= strchr(value,'!');
		ptype = next?atoi(next+1):0;

		// bounds checking
		if(bpart<2 || bpart>29)
			return;
		if(ptype<0 || ptype>255)
			return;

		*(pGameObject+0x0c20+bpart) = (char)ptype;
	}
	
}

void CNWNXFunction::GetBodyPart(char *value) 
{
	int bpart;

	if (*(pGameObject+0x8) == 0x5) // object type creature
	{
		// parse "nPart"
		bpart = atoi(value);

		// bounds checking
		if(bpart<3 || bpart>29)
			return;

		sprintf(value,"%d",*(pGameObject+0x0c20+bpart));
	}
	
}



// apply one long body part byte vector
void CNWNXFunction::SetBodyAll(char *value) {
	int i;
	char buf[3];
	buf[2]= 0;

	if (*(pGameObject+0x8) == 0x5 && value != NULL) // object type creature
	{
		for(i=0; i<30; ++i)
		{
			buf[0]= value[i*2];
			buf[1]= value[i*2+1];
			*(pGameObject+0x0c20+i) = (char)strtoul(buf,NULL,16);
		}
	}
}

// retrieve one long body part byte vector
void CNWNXFunction::GetBodyAll(char *value) {
	
	int i;

	// Check:
	// - is creature object
	// - value != NULL
	// - value can hold the string (60chars)
	if (*(pGameObject+0x8) == 0x5 && value != NULL && strlen(value)>=60)
	{
		for(i=0; i<30; ++i)
		{
			sprintf(&value[i*2],"%02x",*(pGameObject+0x0c20+i));
		}
		value[i*2]= 0;

		// Log(1,"GetBodyAll: value=%s\n",value);
	} else if(*(pGameObject+0x8) != 0x5) {
		// Log(1,"GetBodyAll: Object type %d != 5\n",*(pGameObject+0x8));
	} else if(value == NULL) {
		// Log(1,"GetBodyAll: value=NULL\n");
	} else if(strlen(value)<60) {
		// Log(1,"GetBodyAll: value too short, %d<60\n",strlen(value));
	}
}

char* CNWNXFunction::OnRequest (char *gameObject, char* Request, char* Parameters)
{
	if(gameObject<(char*)0x1000) return NULL;

	this->pGameObject = gameObject;

	if (strncmp(Request, "SETLOCKDC", 9) == 0) 	
	{
		SetLockDC(Parameters);
		return NULL;
	}
	else if (strncmp(Request, "SETGOLDPIECEVALUE", 17) == 0) 	
	{
		SetGoldPieceValue(Parameters);
		return NULL;
	}
	else if (strncmp(Request, "SETTAG", 6) == 0) 	
	{
		SetTag(Parameters);
		return NULL;
	}
	else if (strncmp(Request, "SETARMORAC", 10) == 0) 	
	{
		SetArmorAC(Parameters);
		return NULL;
	}
	else if (strncmp(Request, "GETARMORAC", 10) == 0) 	
	{
		GetArmorAC(Parameters);
		return NULL;
	}
	else if (strncmp(Request, "GETDESCRIPTION", 14) == 0) 	
	{
		GetDescription(Parameters);
		return NULL;
	}
	else if (strncmp(Request, "GETISPICKPOCKETABLE", 19) == 0) 	
	{
		GetIsPickPocketable(Parameters);
		return NULL;
	}
	else if (strncmp(Request, "SETISPICKPOCKETABLE", 19) == 0) 	
	{
		SetIsPickPocketable(Parameters);
		return NULL;
	}

	else if (strncmp(Request, "GETBODYPART",11) == 0) 	
	{
		GetBodyPart(Parameters);
		return NULL;
	}
	else if (strncmp(Request, "SETBODYPART",11) == 0) 	
	{
		SetBodyPart(Parameters);
		return NULL;
	}
	else if (strncmp(Request, "GETBODYALL",10) == 0) 	
	{
		GetBodyAll(Parameters);
		return NULL;
	}
	else if (strncmp(Request, "SETBODYALL",10) == 0) 	
	{
		SetBodyAll(Parameters);
		return NULL;
	}
	
	return NULL;
}

BOOL CNWNXFunction::OnRelease ()
{
	// call base class
	return CNWNXBase::OnRelease ();
}
